/* tslint:disable */
/* eslint-disable */
/**
 * FastAPI
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 0.1.0
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */



/**
 * 
 * @export
 * @interface GetEmbedding
 */
export interface GetEmbedding {
    /**
     * 
     * @type {Array<any>}
     * @memberof GetEmbedding
     */
    'ids'?: Array<any>;
    /**
     * 
     * @type {object}
     * @memberof GetEmbedding
     */
    'where'?: object;
    /**
     * 
     * @type {object}
     * @memberof GetEmbedding
     */
    'where_document'?: object;
    /**
     * 
     * @type {string}
     * @memberof GetEmbedding
     */
    'sort'?: string;
    /**
     * 
     * @type {number}
     * @memberof GetEmbedding
     */
    'limit'?: number;
    /**
     * 
     * @type {number}
     * @memberof GetEmbedding
     */
    'offset'?: number;
    /**
     * 
     * @type {Array<string>}
     * @memberof GetEmbedding
     */
    'include'?: Array<GetEmbeddingIncludeEnum>;
}

/**
    * @export
    * @enum {string}
    */
export enum GetEmbeddingIncludeEnum {
    Documents = 'documents',
    Embeddings = 'embeddings',
    Metadatas = 'metadatas',
    Distances = 'distances'
}


