/* tslint:disable */
/* eslint-disable */
/**
 * FastAPI
 * No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)
 *
 * The version of the OpenAPI document: 0.1.0
 * 
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */



/**
 * 
 * @export
 * @interface AddEmbedding
 */
export interface AddEmbedding {
    /**
     * 
     * @type {Array<any>}
     * @memberof AddEmbedding
     */
    'embeddings': Array<any>;
    /**
     * 
     * @type {Array<any> | object}
     * @memberof AddEmbedding
     */
    'metadatas'?: Array<any> | object;
    /**
     * 
     * @type {string | Array<any>}
     * @memberof AddEmbedding
     */
    'documents'?: string | Array<any>;
    /**
     * 
     * @type {string | Array<any>}
     * @memberof AddEmbedding
     */
    'ids'?: string | Array<any>;
    /**
     * 
     * @type {boolean}
     * @memberof AddEmbedding
     */
    'increment_index'?: boolean;
}

